@@ -10,6 +10,7 @@ from cola import errors
 from cola import utils
 from cola import version
 from cola.compat import set
+from cola.decorators import memoize
 
 git = git.instance()
 config = gitcfg.instance()
@@ -18,18 +19,21 @@ class InvalidRepositoryError(StandardError):
     pass
 
 
-def default_remote():
+def default_remote(config=None):
     """Return the remote tracked by the current branch."""
+    if config is None:
+        config = gitcfg.instance()
     return config.get('branch.%s.remote' % current_branch())
 
 
-def diff_filenames(arg):
+def diff_filenames(arg, git=git):
     """Return a list of filenames that have been modified"""
-    diff_zstr = git.diff(arg, name_only=True, z=True).rstrip('\0')
+    diff_zstr = git.diff(arg, name_only=True, z=True,
+                         **_common_diff_opts()).rstrip('\0')
     return [core.decode(f) for f in diff_zstr.split('\0') if f]
 
 
-def all_files():
+def all_files(git=git):
     """Return the names of all files in the repository"""
     return [core.decode(f)
             for f in git.ls_files(z=True)
@@ -41,7 +45,12 @@ class _current_branch:
     key = None
     value = None
 
-def current_branch():
+
+def clear_cache():
+    _current_branch.key = None
+
+
+def current_branch(git=git):
     """Return the current branch"""
     head = git.git_path('HEAD')
     try:
@@ -67,7 +76,7 @@ def current_branch():
     return data
 
 
-def _read_git_head(head, default='master'):
+def _read_git_head(head, default='master', git=git):
     """Pure-python .git/HEAD reader"""
     # Legacy .git/HEAD symlinks
     if os.path.islink(head):
@@ -101,14 +110,14 @@ def branch_list(remote=False):
         return for_each_ref_basename('refs/heads')
 
 
-def for_each_ref_basename(refs):
+def for_each_ref_basename(refs, git=git):
     """Return refs starting with 'refs'."""
     output = git.for_each_ref(refs, format='%(refname)').splitlines()
     non_heads = filter(lambda x: not x.endswith('/HEAD'), output)
     return map(lambda x: x[len(refs) + 1:], non_heads)
 
 
-def all_refs(split=False):
+def all_refs(split=False, git=git):
     """Return a tuple of (local branches, remote branches, tags)."""
     local_branches = []
     remote_branches = []
@@ -128,8 +137,10 @@ def all_refs(split=False):
         return local_branches + remote_branches + tags
 
 
-def tracked_branch(branch=None):
+def tracked_branch(branch=None, config=None):
     """Return the remote branch associated with 'branch'."""
+    if config is None:
+        config = gitcfg.instance()
     if branch is None:
         branch = current_branch()
     remote = config.get('branch.%s.remote' % branch)
@@ -144,7 +155,7 @@ def tracked_branch(branch=None):
     return None
 
 
-def untracked_files():
+def untracked_files(git=git):
     """Returns a sorted list of untracked files."""
     ls_files = git.ls_files(z=True,
                             others=True,
@@ -159,7 +170,7 @@ def tag_list():
     return tags
 
 
-def commit_diff(sha1):
+def commit_diff(sha1, git=git):
     commit = git.show(sha1)
     first_newline = commit.index('\n')
     if commit[first_newline+1:].startswith('Merge:'):
@@ -171,6 +182,33 @@ def commit_diff(sha1):
         return core.decode(commit)
 
 
+def _common_diff_opts(config=config):
+    patience = version.check('patience', version.git_version())
+    submodule = version.check('diff-submodule', version.git_version())
+    return {
+        'patience': patience,
+        'submodule': submodule,
+        'no_color': True,
+        'no_ext_diff': True,
+        'with_raw_output': True,
+        'with_stderr': True,
+        'unified': config.get('gui.diffcontext', 3),
+    }
+
+
+def sha1_diff(sha1, git=git):
+    return git.diff(sha1 + '^!', **_common_diff_opts())
+
+
+def diff_info(sha1, git=git):
+    log = git.log('-1',
+                  '--format=Author:\t%aN <%aE>%n'
+                  'Date:\t%aD%n%n'
+                  '%s%n%n%b',
+                  sha1)
+    return log + '\n\n' + sha1_diff(sha1)
+
+
 def diff_helper(commit=None,
                 branch=None,
                 ref=None,
@@ -179,7 +217,8 @@ def diff_helper(commit=None,
                 cached=True,
                 with_diff_header=False,
                 suppress_header=True,
-                reverse=False):
+                reverse=False,
+                git=git):
     "Invokes git diff on a filepath."
     if commit:
         ref, endref = commit+'^', commit
@@ -210,16 +249,8 @@ def diff_helper(commit=None,
     patience = version.check('patience', version.git_version())
     submodule = version.check('diff-submodule', version.git_version())
 
-    diffoutput = git.diff(R=reverse,
-                          M=True,
-                          no_color=True,
-                          cached=cached,
-                          unified=config.get('diff.context', 3),
-                          with_raw_output=True,
-                          with_stderr=True,
-                          patience=patience,
-                          submodule=submodule,
-                          *argv)
+    diffoutput = git.diff(R=reverse, M=True, cached=cached,
+                          *argv, **_common_diff_opts())
     # Handle 'git init'
     if diffoutput.startswith('fatal:'):
         if with_diff_header:
@@ -342,7 +373,7 @@ def worktree_state(head='HEAD', staged_only=False):
            state.get('upstream_changed', []))
 
 
-def worktree_state_dict(head='HEAD', staged_only=False):
+def worktree_state_dict(head='HEAD', staged_only=False, git=git):
     """Return a dict of files in various states of being
 
     :rtype: dict, keys are staged, unstaged, untracked, unmerged,
@@ -428,7 +459,8 @@ def worktree_state_dict(head='HEAD', staged_only=False):
     if tracked:
         try:
             diff_expr = merge_base_to(tracked)
-            output = git.diff(diff_expr, name_only=True, z=True)
+            output = git.diff(diff_expr, name_only=True, z=True,
+                              **_common_diff_opts())
 
             if output.startswith('fatal:'):
                 raise errors.GitInitError('git init')
@@ -456,7 +488,7 @@ def worktree_state_dict(head='HEAD', staged_only=False):
             'submodules': submodules}
 
 
-def _branch_status(branch):
+def _branch_status(branch, git=git):
     """
     Returns a tuple of staged, unstaged, untracked, and unmerged files
 
@@ -465,9 +497,9 @@ def _branch_status(branch):
     """
     status, output = git.diff(name_only=True,
                               M=True, z=True,
-                              with_stderr=True,
                               with_status=True,
-                              *branch.strip().split())
+                              *branch.strip().split(),
+                              **_common_diff_opts())
     if status != 0:
         return {}
 
@@ -489,11 +521,12 @@ def merge_base_parent(branch):
     return 'master..%s' % branch
 
 
-def is_modified(name):
+def is_modified(name, git=git):
     status, out = git.diff('--', name,
                            name_only=True,
                            exit_code=True,
-                           with_status=True)
+                           with_status=True,
+                           **_common_diff_opts())
     return status != 0
 
 
@@ -502,20 +535,20 @@ def eval_path(path):
     if path.startswith('"') and path.endswith('"'):
         return core.decode(eval(path))
     else:
-        return path
+        return core.decode(path)
 
 
-def renamed_files(start, end):
-    difflines = git.diff('%s..%s' % (start, end),
-                         no_color=True,
-                         M=True).splitlines()
+def renamed_files(start, end, git=git):
+    difflines = git.diff('%s..%s' % (start, end), M=True,
+                         **_common_diff_opts()).splitlines()
     return [eval_path(r[12:].rstrip())
                 for r in difflines if r.startswith('rename from ')]
 
 
-def changed_files(start, end):
+def changed_files(start, end, git=git):
     zfiles_str = git.diff('%s..%s' % (start, end),
-                          name_only=True, z=True).strip('\0')
+                          name_only=True, z=True,
+                          **_common_diff_opts()).strip('\0')
     return [core.decode(enc) for enc in zfiles_str.split('\0') if enc]
 
 
