@@ -115,6 +115,8 @@ def main():
     from cola import qtutils
     from cola import cmds
 
+    # TODO: remove in 2012?
+    has_threadpool = hasattr(QtCore, 'QThreadPool')
 
     # Allow Ctrl-C to exit
     signal.signal(signal.SIGINT, signal.SIG_DFL)
@@ -165,18 +167,12 @@ def main():
     # Finally, go to the root of the git repo
     os.chdir(model.git.worktree())
 
-    # Show the GUI and start the event loop
+    # Show the GUI
     if opts.classic:
         view = cola_classic(update=False)
     else:
         view = MainView()
-        MainController(model, view)
-
-    # Scan for the first time
-    model.update_status()
-
-    # Start the inotify thread
-    inotify.start()
+        ctl = MainController(model, view)
 
     # Show the view and start the main event loop
     view.show()
@@ -184,6 +180,15 @@ def main():
     # Make sure that we start out on top
     view.raise_()
 
+    # Scan for the first time
+    if has_threadpool:
+        task = _start_update_thread(model)
+    else:
+        model.update_status()
+
+    # Start the inotify thread
+    inotify.start()
+
     git.GIT_COLA_TRACE = os.getenv('GIT_COLA_TRACE', False)
     if git.GIT_COLA_TRACE:
         msg = ('info: Trace enabled.  '
@@ -191,13 +196,14 @@ def main():
                '"plumbing" API and are not intended for typical '
                'day-to-day use.  Here be dragons')
         cola.notifier().broadcast(signals.log_cmd, 0, msg)
+
+    # Start the event loop
     result = app.exec_()
 
     # All done, cleanup
     inotify.stop()
 
-    # TODO: Provide fallback implementation
-    if hasattr(QtCore, 'QThreadPool'):
+    if has_threadpool:
         QtCore.QThreadPool.globalInstance().waitForDone()
 
     pattern = cola.model().tmp_file_pattern()
@@ -206,6 +212,26 @@ def main():
     sys.exit(result)
 
 
+def _start_update_thread(model):
+    """Update the model in the background
+
+    git-cola should startup as quickly as possible.
+
+    """
+    from PyQt4 import QtCore
+
+    class UpdateTask(QtCore.QRunnable):
+        def run(self):
+            model.update_status()
+
+    # Hold onto a reference to prevent PyQt from dereferencing
+    task = UpdateTask()
+    QtCore.QThreadPool.globalInstance().start(task)
+
+    return task
+
+
+
 def _setup_resource_dir(dirname):
     """Adds resource directories to Qt's search path"""
     from cola import qtcompat
